


long_name = 'HumPack'
name = 'humpack'

version = '0.1.1'

description = 'Human Readable Object Serialization and more'

url = 'https://github.com/fleeb24/HumPack'

author = 'Felix Leeb'
author_email = 'felix.leeb@hotmail.com'

license = 'GPL3'

packages = ['humpack']

# Automatically get list of requirements from requirements.txt
import os
path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
install_requires = []
if 'requirements.txt' not in os.listdir(path):
    print('WARNING: no requirements.txt found')
    install_requires = ['wrapt', 'cryptography']
    # raise FileNotFoundError('requirements.txt not found, this usually happens if part of this library is missing')
else:
    with open(os.path.join(path, 'requirements.txt'), 'r') as f:
        install_requires = [pk[:-1] for pk in f.readlines() if len(pk)]






from .packing import Packable, primitive, PRIMITIVE, SERIALIZABLE, JSONABLE, pack, unpack, pack_member, unpack_member
from .packing import save_pack, load_pack, json_pack, json_unpack
from .transactions import Transactionable, AbortTransaction
from .hashing import Hashable

# from .wrappers import ObjectWrapper, Array # causes an error if required libs aren't already installed
try:
	import numpy as _numpy
except ImportError:
	pass
else: # Register additional common packable types
	from . import common

from .basic_containers import tdict, tlist, tset, tstack, tdeque, theap
from .basic_containers import containerify

from ._lib_info import version as __version__
from ._lib_info import author as __author__
from ._lib_info import path as __home__




from typing import Any, Union, Dict, List, Set, Tuple, NoReturn, ClassVar, TextIO, Callable

import base64
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC

from .packing import json_pack, json_unpack, SERIALIZABLE, PACKED, JSONABLE
from .errors import WrongKeyError
from cryptography.fernet import Fernet, InvalidToken, InvalidSignature
from crypt import crypt, mksalt
from getpass import getpass
# from hmac import compare_digest


_master_salt = '6FwLrxJb5mTPVwthumpackMASTERsalt'

def format_key(hsh: Union[str, bytes]) -> bytes:
	'''
	Reformat a hash (can be str or bytes)
	
	:param hsh: hash to be reformatted
	:return: a key which can be used for decrypting the data
	'''
	if isinstance(hsh, str):
		hsh = hsh.encode('latin1')
	
	kdf = PBKDF2HMAC(
		algorithm=hashes.SHA256(),
		length=32,
		salt=_master_salt.encode('latin1'),
		iterations=100000,
		backend=default_backend(),
	)
	return base64.urlsafe_b64encode(kdf.derive(hsh))

def secure_key(word: str, salt: str = None) -> str:
	'''
	Get a hash from the raw text password
	
	:param word: raw text password
	:param salt: random salt to seed the hash computation
	:return: hash of the password
	'''
	if salt is None:
		salt = _master_salt
	hsh = crypt(word, salt)
	return hsh

def prompt_password_hash(salt: str = None) -> str:
	'''
	Get the hash of a password which is entered by the user in a prompt
	
	:param salt: random salt for hash computation
	:return: hash of the entered password
	'''
	if salt is None:
		salt = _master_salt
	hsh = secure_key(getpass(), salt=salt)
	return hsh



def encrypt(data: bytes, hsh: Union[str, bytes] = None) -> bytes:
	'''
	Encrypt the data from a provided hash (or prompt for a password if no hash is provided)
	
	:param data: bytes to be encrypted
	:param hsh: hash used as key to encrypt
	:return: encrypted data bytes
	'''
	if hsh is None:
		hsh = prompt_password_hash()
	key = format_key(hsh)
	f = Fernet(key)
	return f.encrypt(data)

def decrypt(data: bytes, hsh: Union[str, bytes] = None) -> bytes:
	'''
	Decrypt data with provided hash (or prompt for a password to compute hash if no hash is provided)
	
	:param data: encrypted bytes that should be decrypted
	:param hsh: hash of a password to be used as a key to decrypt (obviously, must be the same as was used to encrypt)
	:return: the decrypted data, or a WrongKeyError if the key failed
	'''
	if hsh is None:
		hsh = prompt_password_hash()
	key = format_key(hsh)
	f = Fernet(key)
	try:
		return f.decrypt(data)
	except (InvalidToken, InvalidSignature):
		pass
	raise WrongKeyError



def secure_pack(obj: SERIALIZABLE, hsh: Union[str, bytes] = None,
                meta: Dict[str,JSONABLE] = None, include_timestamp:bool = False) -> bytes:
	'''
	Pack the object and encrypt it using the provided hash (prompt user for password if none is provided)
	
	:param obj: object to be packed
	:param hsh: hash used as key to encrypt
	:param meta: meta information to store with the packed `obj`
	:param include_timestamp: include timestamp in meta info
	:return: encrypted bytes
	'''
	data = json_pack(obj, meta=meta, include_timestamp=include_timestamp)
	data = data.encode('latin1')

	if hsh is None:
		hsh = prompt_password_hash()

	return encrypt(data, hsh)

def secure_unpack(data: bytes, hsh: Union[str, bytes] = None,
                  return_meta: bool = False) -> SERIALIZABLE:
	'''
	Decrypt `data` and unpack to recover the original object using the provided hash as a key
	
	:param data: encrypted bytes
	:param hsh: hash to be used as a key to decrypt (prompt user, if not provided)
	:param return_meta: include meta info in output
	:return: decrypted and unpacked object, possibly including the meta info
	'''
	if hsh is None:
		hsh = prompt_password_hash()
	
	data = decrypt(data, hsh).decode('latin1')
	data = json_unpack(data, return_meta=return_meta)

	return data








import os
from setuptools import setup
from humpack import _lib_info as info

with open('README.rst', 'r') as f:
      lines = f.readlines()

readme = []
valid = False
for line in lines:
      if valid:
            if 'end-setup-marker-do-not-remove' in line:
                  valid = False
            else:
                  readme.append(line)
      elif 'setup-marker-do-not-remove' in line:
            valid = True

README = '\n'.join(readme)


setup(name=info.name,
      version=info.version,
      description=info.description,
      long_description=README,
      url=info.url,
      author=info.author,
      author_email=info.author_email,
      license=info.license,
      packages=info.packages,
      install_requires=info.install_requires,
      zip_safe=False)
